/*
 * Card.java
 *
 * Created on 28. April 2007, 11:14
 */

package hexenhexen;

/**
 *
 * @author Sunshine
 */
public class Card {
    
    // define the possible sides
    // note that two sides match if their difference is absolute(4)
    public static final byte RED_FRONT = 0;
    public static final byte YELLOW_FRONT = 1;
    public static final byte GREEN_FRONT = 2;
    public static final byte PURPLE_FRONT = 3;
    
    public static final byte RED_BACK = 4;
    public static final byte YELLOW_BACK = 5;
    public static final byte GREEN_BACK = 6;
    public static final byte PURPLE_BACK = 7;
    
    public static final byte UNDEFINED_SIDE = 30;
    
    // properties of this card
    public byte[] sides;           // what is has on each side
    public int rotationPos;        // how many degrees the card is rotated (0, 90, 180, 270)
    
    public boolean isPlaced;       // true if this card is already used (= lies on the game field)
    
    
    public Card()
    {
        isPlaced = false;
        sides = new byte[4];
        rotationPos = 0;
        
        sides[0] = UNDEFINED_SIDE;
        sides[1] = UNDEFINED_SIDE;
        sides[2] = UNDEFINED_SIDE;
        sides[3] = UNDEFINED_SIDE;
    }
    
    
    /** Creates a new instance of Card */
    public Card(byte top, byte right, byte bottom, byte left) {
        isPlaced = false;
        sides = new byte[4];
        rotationPos = 0;
        
        sides[0] = top;
        sides[1] = right;
        sides[2] = bottom;
        sides[3] = left;
    }
    
    // print card to console
    public void printCard()
    {
        int temp = 4 - this.rotationPos / 90;
        System.out.println("Top = " + sideToString(sides[(0+temp) % 4]) );
        System.out.println("Right = " + sideToString(sides[(1+temp) % 4]) );
        System.out.println("Bottom = " + sideToString(sides[(2+temp) % 4]) );
        System.out.println("Left = " + sideToString(sides[(3+temp) % 4]) );
        System.out.println("--------------");
    }
    
    // convert the side value to string
    public String sideToString(byte sideCode)
    {
        switch (sideCode)
        {
            case RED_FRONT: return "Red Front";
            case GREEN_FRONT: return "Green Front";
            case YELLOW_FRONT: return "Yellow Front";
            case PURPLE_FRONT: return "Purple Front";
            
            case RED_BACK: return "Red Back";
            case GREEN_BACK: return "Green Back";
            case YELLOW_BACK: return "Yellow Back";
            case PURPLE_BACK: return "Purple Back";
            
            case UNDEFINED_SIDE: return "UNDEFINED";
            
            default: return "";
        }
    }
    
    /**
     * Set the rotation from the inital position, clockwise in 0, 90, 180 or 270 degress.
     */
    public void setRotate(int rot)
    {
        if (!(rot == 0 || rot == 90 || rot == 180 || rot == 270)) return;
        this.rotationPos = rot;
    }
    
    // returns the value at the right side of the card
    public byte rightSide()
    {
        return sides[(1 + (4 - this.rotationPos / 90)) % 4];
    }
    
    // returns the value at the top side of the card
    public byte topSide()
    {
        return sides[(0 + (4 - this.rotationPos / 90)) % 4];
    }
    
    // returns the value at the bottom side of the card
    public byte bottomSide()
    {
        return sides[(2 + (4 - this.rotationPos / 90)) % 4];
    }
    
    // returns the value at the left side of the card
    public byte leftSide()
    {
        return sides[(3 + (4 - this.rotationPos / 90)) % 4];
    }
    
}
