/* Sunshine's backtracking to little cardgame "Hexen hexen".
 * For more information about this game for kids, have a look at my website.
 *
 * It finds 8 solutions in about 150ms on my laptop, but in fact this game has just
 * two different solutions. So 4 solutions are always the same cause normally it makes no
 * difference from which side you look at the filled 3x3 square but for this little tool,
 * this makes a difference (stores it in a 2dim array).
 *
 * www.sunshine2k.de
 * webmaster@sunshine2k.de
 *
 * Main.java
 *
 * Created on 28. April 2007, 11:11
 *
 **/

package hexenhexen;

/**
 *
 * @author Sunshine
 */
public class Main {

    public Card[] cards;        // the 9 avalaible cards

    /*  (0,0) | (0,1) | (0,2)
     *  (1,0) | (1,1) | (1,2)
     *  (2,0) | (2,1) | (2,2)
     */
    public Card[][] gameField;  // the 3x3 square card field

    public int numOfSolutions;  // number of averall found solutions
    public int posChecked;

    /** Creates a new instance of Main */
    public Main() {
        cards = new Card[9];
        gameField = new Card[3][3];

        // fill in the given cards
        cards[0] = new Card(Card.GREEN_BACK, Card.RED_BACK, Card.PURPLE_FRONT, Card.YELLOW_FRONT);
        cards[1] = new Card(Card.GREEN_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT, Card.RED_FRONT);
        cards[2] = new Card(Card.RED_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT, Card.YELLOW_FRONT);
        cards[3] = new Card(Card.YELLOW_BACK, Card.RED_BACK, Card.GREEN_FRONT, Card.PURPLE_FRONT);
        cards[4] = new Card(Card.RED_BACK, Card.PURPLE_BACK, Card.GREEN_FRONT, Card.PURPLE_FRONT);
        cards[5] = new Card(Card.PURPLE_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT, Card.RED_FRONT);
        cards[6] = new Card(Card.YELLOW_FRONT, Card.PURPLE_BACK, Card.RED_BACK, Card.PURPLE_FRONT);
        cards[7] = new Card(Card.RED_FRONT, Card.GREEN_BACK, Card.YELLOW_BACK, Card.PURPLE_FRONT);
        cards[8] = new Card(Card.GREEN_FRONT, Card.RED_BACK, Card.PURPLE_BACK, Card.PURPLE_FRONT);
    }


    // print solution
    public void printSolution()
    {
        System.out.println("Solution Nr. " + this.numOfSolutions);
        for (int i = 0; i < 3; i++)
            for (int j = 0; j < 3; j++)
            {
                System.out.println("CARD ["+ i + "][" + j + "]");
                gameField[i][j].printCard();
            }
        System.out.println("==================================");
    }


    // returns true if both given side matches
    // have a look how I defined the possible sides of the cards to understand it
    public boolean sidesMatches(byte sideOne, byte sideTwo)
    {
        return (Math.abs(sideOne - sideTwo) == 4);
    }

    // check whether card can be placed @ gamefield[row][column]
    // note that we fill in the cards from left to right and from top to bottom
    public boolean IsPlacingValid(int row, int column, Card card)
    {
        // first card on field is always good :-)
        if ( (row == 0) && (column == 0)) return true;

        if ( (row == 0) && (column == 1))
        {
            return (sidesMatches(card.leftSide(), gameField[0][0].rightSide()));
        }

        if ( (row == 0) && (column == 2))
        {
            if (sidesMatches(card.leftSide(), gameField[0][1].rightSide()))
                return true;
        }

        if ( (row == 1) && (column == 0))
        {
            return ( sidesMatches(card.topSide(), gameField[0][0].bottomSide()));
        }

        if ( (row == 1) && (column == 1))
        {
            return  ( sidesMatches(card.topSide(), gameField[0][1].bottomSide()) &&
                      sidesMatches(card.leftSide(), gameField[1][0].rightSide()) );
        }

        if ( (row == 1) && (column == 2))
        {
            return ( sidesMatches(card.topSide(), gameField[0][2].bottomSide()) &&
                     sidesMatches(card.leftSide(), gameField[1][1].rightSide()) );
        }

        if ( (row == 2) && (column == 0))
        {
            return ( sidesMatches(card.topSide(), gameField[1][0].bottomSide()));
        }

        if ( (row == 2) && (column == 1))
        {
            return ( sidesMatches(card.topSide(), gameField[1][1].bottomSide()) &&
                     sidesMatches(card.leftSide(), gameField[2][0].rightSide()) );
        }

        if ( (row == 2) && (column == 2) )
        {
            return ( sidesMatches(card.topSide(), gameField[1][2].bottomSide()) &&
                     sidesMatches(card.leftSide(), gameField[2][1].rightSide()) );
        }
        return false;
    }


    // Solve it :-)
    public void Solve() {
        // re-init gamefield
        gameField = new Card[3][3];
        for (int i = 0; i < 3; i++)
            for (int j = 0; j < 3; j++)
                gameField[i][j] = new Card();
        numOfSolutions = 0;
        posChecked = 0;
        Solve(0);
    }


    // the internal, recursively called solve function
    private void Solve(int step)
    {
        int row = step / 3;
        int column = step % 3;

        // try all available cards
        for (int i = 0; i < 9; i++)
        {
            // is this card is already placed on the field, go on with next card
            if (cards[i].isPlaced) continue;

            posChecked++;

            // check for each card all four possible rotated positions
            for (int rotPos = 0; rotPos < 360; rotPos += 90)
            {
                cards[i].setRotate(rotPos);

                // check if the card can be placed
                if (IsPlacingValid(row, column, cards[i]))
                {
                    // place card
                    cards[i].isPlaced = true;
                    setCard(row, column, cards[i]);

                    // we set 9 cards, so solution found
                    if (step >= 8)
                    {
                        numOfSolutions++;
                        printSolution();
                    }
                    else    // try setting next card
                        Solve(step+1);          // NOT ++step!!!!

                    // remove card
                    cards[i].isPlaced = false;
                    removeCard(row, column);
                }
            }   // end rotation loop
            cards[i].setRotate(0);
        }
    }

    // set given card at gamefield[row][column]
    public void setCard(int row, int column, Card card)
    {
        gameField[row][column] = card;
    }

    // remove card at gamefield[row][column]
    public void removeCard(int row, int column)
    {
        gameField[row][column] = new Card();
    }



    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Main MainGame = new Main();
        long timeDiff = System.currentTimeMillis();
        MainGame.Solve();
        timeDiff = System.currentTimeMillis() - timeDiff;
        System.out.println("Found Solutions: " + MainGame.numOfSolutions);
        System.out.println("Time needed: " + timeDiff + " ms");
        System.out.println("posChecked: " + MainGame.posChecked);
    }

}
