/*
 * Grid.java
 *
 * Created on 25. Juni 2007, 18:05
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.awt.*;

/**
 *
 * @author Sunshine
 */
public class Grid {
    
    private int x;              // x position of grid on applet
    private int y;              // y position of grid on applet
    private int blocksize;      // size of one quadratic gridblock
    
    // save on which gridblock a card is placed
    private boolean[][] blockOccupied = new boolean[3][3];
    
    // save the position of the upperleft corner of every gridblock
    private Point[][] hotcorners = new Point[3][3];
    
    /** Creates a new instance of Grid */
    public Grid(int xPos, int yPos, int size) {
        x = xPos;
        y = yPos;
        blocksize = size;
        
        // calculate the upperleft corner of each gridblock
        for (int i = 0; i < 3; i++)
            for (int j = 0; j < 3; j++)
            {
                hotcorners[i][j] = new Point(x + (i*blocksize), y + j*blocksize);
                // no card is placed on startup
                blockOccupied[i][j] = false;
            }
    }
    
    // draw the grid
    public void draw(Graphics g)
    {
        for (int i = 0; i < 4; i++)
        {
            // vertical line
            g.drawLine(x + i * blocksize, 10, x + i * blocksize, 10 + 3 * blocksize);
            // horizontal line
            g.drawLine(x, 10 + i * blocksize,  x + 3 * blocksize, 10 + i * blocksize);
        }
    }
    
    // returns the new position of the just dragged card or null if position didn't change
    public Point getNewPos(int x, int y, CardEx card)
    {
        for (int i = 0; i < 3; i++)
            for (int j = 0; j < 3; j++)
            {
                if (x >= hotcorners[i][j].x && x < hotcorners[i][j].x + blocksize &&
                    y >= hotcorners[i][j].y && y < hotcorners[i][j].y + blocksize)
                {
                    if (blockOccupied[i][j])  
                    {
                        // there is already a card on the block, so move card back to its original position
                        
                        // special handling if card has layed on the grid
                        if (card.isPlaced && card.getX() >= this.x && card.getY() >= this.y)
                        {
                            blockOccupied[card.gridBlockX][card.gridBlockY] = false;
                            card.gridBlockX = -1;
                            card.gridBlockY = -1;
                            card.isPlaced = false;
                        }
                        return new Point(card.xOriginal, card.yOriginal);
                    }
                    else
                    {
                        if (card.gridBlockX >= 0 && card.gridBlockY >= 0)
                            blockOccupied[card.gridBlockX][card.gridBlockY] = false;
                        blockOccupied[i][j] = true;
                        card.gridBlockX = i;
                        card.gridBlockY = j;
                        card.isPlaced = true;
                        return new Point(hotcorners[i][j].x, hotcorners[i][j].y);
                    }
                }
            }
        
        // here the card was NOT dragged on the grid; perhaps it was on the grid and was dragged outside the grid
        // then it should be moved to its original pos
        if (card.isPlaced)
        {
            blockOccupied[card.gridBlockX][card.gridBlockY] = false;
            card.gridBlockX = -1;
            card.gridBlockY = -1;
            card.isPlaced = false;
            return new Point(card.xOriginal, card.yOriginal);
        }
        return null;
    }

    // returns true if on all 9 gridblocks a card is placed
    boolean AllPlacesOccupied() 
    {
        for (int i = 0; i < 3; i++)
            for (int j = 0; j < 3; j++)
            {
                if (!blockOccupied[i][j])
                    return false;
            }
        return true;
    }
    
}
