/*
 * HexenApplet.java
 * Author : Sunshine
 * Web: www.sunshine2k.de
 * Created on 25. Mai 2007, 19:27
 */
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.*;
import java.net.*;

/**
 *
 * @author  Sunshine
 */
public class HexenApplet extends /*javax.swing.JApplet*/ java.applet.Applet implements MouseMotionListener, MouseListener {
    
    public static final int WIDTH = 775;           // size of applet
    public static final int HEIGHT = 420;
    
    private CardEx cards[];                         // our 9 cards
    private Grid grid;                              // gamefield
    
    private Image offImage = null;                  // offscreen image for double-buffering
    private boolean bGameSolved = false;            // true if game is solved
    
    // info on dragged card
    private boolean isDragging = false;             // true if a card is currently dragged
    private int     draggedCardID = -1;             // ID of card currently dragging
    private int     xOffset = 0, yOffset = 0;       // difference between 'dragpoint' and card's upperleft corner
    private int     oldPosX, oldPosY;               // position of card before dragging
    
    
    /** Initializes the applet HexenApplet */
    public void init() {
        
        // autogenerated
        try {
            java.awt.EventQueue.invokeAndWait(new Runnable() {
                public void run() {
                    //initComponents();
                }
            });
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
        this.setSize(WIDTH, HEIGHT);
        offImage = this.createImage(WIDTH, HEIGHT);
        
        // create cards
        cards = new CardEx[9];
       
        cards[0] = new CardEx(10, 10, Card.GREEN_FRONT, Card.RED_BACK, Card.PURPLE_BACK, Card.PURPLE_FRONT);
        cards[1] = new CardEx(10, 20 + CardEx.IMAGESIZE, Card.PURPLE_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT, Card.RED_FRONT);
        cards[2] = new CardEx(10, 30 + CardEx.IMAGESIZE*2, Card.PURPLE_FRONT, Card.RED_FRONT, Card.GREEN_BACK, Card.YELLOW_BACK);
        cards[3] = new CardEx(20 + CardEx.IMAGESIZE, 10,
                            Card.PURPLE_FRONT, Card.YELLOW_FRONT, Card.PURPLE_BACK, Card.RED_BACK);
        cards[4] = new CardEx(20 + CardEx.IMAGESIZE, 20 + CardEx.IMAGESIZE, 
                            Card.YELLOW_FRONT, Card.RED_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT);
        cards[5] = new CardEx(20 + CardEx.IMAGESIZE, 30 + CardEx.IMAGESIZE*2, 
                            Card.GREEN_BACK, Card.YELLOW_BACK, Card.GREEN_FRONT, Card.RED_FRONT);
        cards[6] = new CardEx(30 + CardEx.IMAGESIZE*2, 10, 
                            Card.GREEN_BACK, Card.RED_BACK, Card.PURPLE_FRONT, Card.YELLOW_FRONT);
        cards[7] = new CardEx(30 + CardEx.IMAGESIZE*2, 20 + CardEx.IMAGESIZE,
                            Card.RED_BACK, Card.PURPLE_BACK, Card.GREEN_FRONT, Card.PURPLE_FRONT);
        cards[8] = new CardEx(30 + CardEx.IMAGESIZE*2, 30 + CardEx.IMAGESIZE*2,
                            Card.YELLOW_BACK, Card.RED_BACK, Card.GREEN_FRONT, Card.PURPLE_FRONT);
        
        // create grid field
        grid = new Grid(3 * (CardEx.IMAGESIZE + 12), 10, CardEx.IMAGESIZE);
        
        // load the hexen images
        LoadImages();
        
        // shuffle start position of the cards - must be after LoadImages() !
        shufflePositions();
        // rotate the cards randomly
        rotateCards();
        
        // config stuff
        this.addMouseMotionListener(this);
        this.addMouseListener(this);
        
        repaint();
    }
    
    /*
     * Shuffle the start positions of the cards randomly
     */
    private void shufflePositions()
    {
        int j = cards.length - 1;
        int k;
        int xpos, ypos, xposOri, yposOri;;
        while (j > 0)
        {
            k = (int)(j * Math.random());
            // swap the position of cards at k and j
            xpos = cards[k].getX();
            ypos = cards[k].getY();
            xposOri = cards[k].xOriginal;
            yposOri = cards[k].yOriginal;
            cards[k].setPos(cards[j].getX(), cards[j].getY());
            cards[k].xOriginal = cards[j].xOriginal;
            cards[k].yOriginal = cards[j].yOriginal;
            cards[j].setPos(xpos, ypos);
            cards[j].xOriginal = xposOri;
            cards[j].yOriginal = yposOri;
            j--;
        }
    }
    
    /*
     * Rotate all cards randomly
     */
    public void rotateCards()
    {
        int random;
        for (int i = 0; i < 9; i++)
        {
            random = ((int)(Math.random() * 4)) * 90;
            cards[i].setRotate(random);
        }
    }
    
    public void update(Graphics g) { paint(g); }
    
    public void paint(Graphics g)
    {
        Graphics2D gOff = (Graphics2D)offImage.getGraphics();
        
        // clear background
        gOff.setColor(Color.WHITE);
        gOff.fillRect(0, 0, WIDTH, HEIGHT);
        gOff.setColor(Color.BLACK);
        
        // draw grid
        grid.draw(gOff);
        
        DrawImages(gOff);
        
        Font font = new Font("Arial", Font.BOLD, 20);
        Font oldFont = gOff.getFont();
        Color oldColor = gOff.getColor();
        gOff.setFont(font);
        if (this.bGameSolved)
        {
            gOff.setColor(Color.GREEN);
            gOff.drawString("You solved it!", 420, 400);
        }
        else
        {
            gOff.setColor(Color.BLUE);
            gOff.drawString("Keep on trying...", 420, 400); 
        }
        gOff.setColor(oldColor);
        gOff.setFont(oldFont);
        
        g.drawImage(offImage, 0, 0, null);  
    }
    
    /**
     * Load images
     */
    void LoadImages()
    {
        MediaTracker tracker = new MediaTracker(this);
        String s = this.getCodeBase().toString();
        
        Image image;
        for (int i = 0; i < 9; i++)
        {
            try {
                image = this.getImage(new URL(this.getCodeBase()+ "Hexenbilder/" + (i+1) + ".jpg"));
                tracker.addImage(image, 0);
                cards[i].setImage(image);
            }
            catch (MalformedURLException ex) 
            { 
                continue;
            }
        }
        
        try {
                tracker.waitForID(0);
        }
        catch (InterruptedException e) {
                e.printStackTrace();
        }

    }
    
    private void DrawImages(Graphics2D g) {
        for (int i = 0; i < 9; i++)
        {
            cards[i].draw(g);
        }
    }
    
    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    /*
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 846, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 496, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    */
    public void mouseDragged(MouseEvent e) 
    {
        if (draggedCardID == -1) return;
        cards[draggedCardID].setPos(e.getPoint().x - this.xOffset, e.getPoint().y - this.yOffset);
        repaint();
    }

    public void mousePressed(MouseEvent e) {
      
        // check if we clicked a card
        for (int i = 0; i < 9; i++)
        {
            if (cards[i].isPointInside(e.getPoint()))
            {
                // check which mouse button was pressed
                if (e.getButton() == MouseEvent.BUTTON1)
                {
                    // left mouse button, so drag it
                    draggedCardID = i;
                    this.isDragging = true;
                    this.xOffset = cards[i].xOffsetToCorner(e.getPoint());
                    this.yOffset = cards[i].yOffsetToCorner(e.getPoint());
                    oldPosX = cards[i].getX();
                    oldPosY = cards[i].getY();
                    break;
                }
                else if (e.getButton() == MouseEvent.BUTTON3)
                {
                    // right mouse button, so rotate image by 90 degrees
                    cards[i].setRotate((cards[i].rotationPos + 90) % 360);
                    repaint();
                }
            }
        } 
    }

    public void mouseReleased(MouseEvent e) {
        // we did not drag anything... bye
        if (draggedCardID == -1) return;
        
        Point p = grid.getNewPos(e.getX(), e.getY(), cards[draggedCardID]);
        if (p != null)
        { 
            cards[draggedCardID].setPos(p);
        }
        else
        {
            // set back to original pos
            cards[draggedCardID].setPos(oldPosX, oldPosY);
        }
        if (CheckSolution())
        {
            // solution found!
            bGameSolved = true;
        }
        repaint();
        isDragging = false;
        draggedCardID = -1;
    }

   
    // returns true if both given side matches
    // have a look how I defined the possible sides of the cards to understand it
    public boolean sidesMatches(byte sideOne, byte sideTwo)
    {
        return (Math.abs(sideOne - sideTwo) == 4);
    }
    
    private boolean CheckSolution() 
    {
        if (!grid.AllPlacesOccupied()) return false;
        
        // now build gamefield
        CardEx[][] field = new CardEx[3][3];
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 3; j++)
            {
                field[cards[i*3 + j].gridBlockX][cards[i*3 + j].gridBlockY] = cards[i*3 + j];
            }
        }
        
        // now check is everything is good :-)
        /*  (0,0) | (1,0) | (2,0)
         *  (0,1) | (1,1) | (2,1)
         *  (0,2) | (1,2) | (2,2)
         */
        return (sidesMatches(field[0][0].rightSide(), field[1][0].leftSide()) &&
                sidesMatches(field[1][0].rightSide(), field[2][0].leftSide()) &&
                sidesMatches(field[0][0].bottomSide(), field[0][1].topSide()) && 
                sidesMatches(field[1][0].bottomSide(), field[1][1].topSide()) &&
                sidesMatches(field[2][0].bottomSide(), field[2][1].topSide()) &&
                sidesMatches(field[0][1].rightSide(), field[1][1].leftSide()) &&
                sidesMatches(field[1][1].rightSide(), field[2][1].leftSide()) &&
                sidesMatches(field[0][1].bottomSide(), field[0][2].topSide()) &&
                sidesMatches(field[1][1].bottomSide(), field[1][2].topSide()) &&
                sidesMatches(field[2][1].bottomSide(), field[2][2].topSide()) &&
                sidesMatches(field[0][2].rightSide(), field[1][2].leftSide()) &&
                sidesMatches(field[1][2].rightSide(), field[2][2].leftSide()) );
        
    }

    
    
    public void mouseMoved(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }
    
    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
